import { Request, Response } from 'express';
import * as path from 'path';
import * as fs from 'fs';
import { BSON } from 'bson';
import { rootPath } from './global';
import { showSaveViewDialog } from './windows';

const payloadCache = new Map<string, any>();

function mergePayloadCache(file: string, payload: any) {
    if (!payloadCache.has(file)) {
        payloadCache.set(file, payload);
    }
    const originPayload = payloadCache.get(file);
    Object.assign(originPayload, payload);
    return originPayload;
}

export async function saveView(req: Request, res: Response) {
    try {
        // 此处的 payload 可以是 partial
        const { originVcdFile, originVcdViewFile, payload } = req.body;
        payload.originVcdFile = path.isAbsolute(originVcdFile) ? originVcdFile : path.join(path.resolve(rootPath), originVcdFile).replace(/\\/g, '/');

        const originPayload = mergePayloadCache(originVcdViewFile, payload);

        const savePath = path.isAbsolute(originVcdViewFile) ? originVcdViewFile : path.join(rootPath, originVcdViewFile);
        const buffer = BSON.serialize(originPayload);
        fs.writeFileSync(savePath, buffer);
        res.send('success');
    } catch (error) {
        console.log('error happen in /save-view, ' + error);
        res.send('error');
    }
}

export async function saveViewAs(req: Request, res: Response) {
    try {
        // 先保存原来的文件 payload 一定是 all
        const { originVcdFile, originVcdViewFile, payload } = req.body;
        payload.originVcdFile = path.isAbsolute(originVcdFile) ? originVcdFile : path.join(path.resolve(rootPath), originVcdFile).replace(/\\/g, '/');

        const originPayload = mergePayloadCache(originVcdViewFile, payload);

        // 询问新的路径
        const savePath = await showSaveViewDialog();

        if (savePath) {
            const buffer = BSON.serialize(originPayload);
            fs.writeFileSync(savePath, buffer);
            
            // 创建新的缓存 savePath 会成为新的 originVcdViewFile
            mergePayloadCache(savePath, payload);

            res.send({
                viewPath: savePath.replace(/\\/g, '/')
            });
        } else {
            res.send({
                viewPath: undefined
            });
        }
    } catch (error) {
        console.log('error happen in /save-view-as, ' + error);
        res.send('error');
    }
}